@@ -1,3 +1,65 @@
+#' Convert Greek to Latex
+#' 
+#' Converts Greek letter names to latex.
+#' By default, assumes latex package \code{upgreek} is available
+#' so that lower case greek can be typeset upright, e.g. using 
+#' backslash-upalpha instead of backslash-alpha. Do use the package
+#' "upgreek" in your document preamble,
+#' or else in your R environment set \code{options(spork_upgreek = FALSE)}.
+#' 
+#' @param x greek
+#' @param ... ignored,
+#' @param upgreek if TRUE, assume latex package \code{upgreek} is available
+#' @family latex
+#' @keywords internal
+#' @export
+#' @return latex
+#' @examples
+#' as_latex(greek())
+as_latex.greek <- function(x, ..., upgreek = getOption('spork_upgreek', TRUE)){
+  stopifnot(all(x %in% greek()))
+  # https://www.overleaf.com/learn/latex/List_of_Greek_letters_and_math_symbols
+
+    y <- x
+  
+  # general substitutions
+  y[y == 'Upsilon1'] <- 'Upsilon'
+  y[y == 'omega1'] <-  'varpi'
+  y[y == 'theta1'] <-  'vartheta'
+  y[y == 'phi1'] <-  'varphi'
+  y[y == 'sigma1'] <-  'varsigma'
+  y[y == 'stigma'] <-  'varsigma'
+  
+  # handle upgreek
+  prefix <- '\\'
+  stopifnot(length(upgreek) == 1, is.logical(upgreek))
+  if(upgreek){
+    prefix <- ifelse(grepl('[A-Z]', y), '\\Up', '\\up')
+    y <- tolower(x) # see documentation for upgreek, e.g. \\Gamma -> \\Upgamma
+  }
+  y <- paste0(prefix, y)
+  
+  # Roman literals not covered by upgreek
+  y[x == 'omicron'] <- 'o'
+  y[x == 'Alpha'] <- 'A'
+  y[x == 'Beta'] <- 'B'
+  y[x == 'Epsilon'] <- 'E'
+  y[x == 'Zeta'] <- 'Z'
+  y[x == 'Eta'] <- 'H'
+  y[x == 'Iota'] <- 'I'
+  y[x == 'Kappa'] <- 'K'
+  y[x == 'Mu'] <- 'M'
+  y[x == 'Nu'] <- 'N'
+  y[x == 'Omicron'] <- 'O'
+  y[x == 'Rho'] <- 'P'
+  y[x == 'Tau'] <- 'T'
+  y[x == 'Chi'] <- 'X'
+  
+  class(y) <- c('latex','character')
+  y
+}
+
+
 #' Coerce to Latex
 #'
 #' Coerce to latex.  Generic, with method
@@ -43,6 +105,7 @@ as_latex <- function(x, ...)UseMethod('as_latex')
 #' @param math_open,math_close these wrap math-like portions of the label;  the defaults try to give upright characters (non-italic) which may not work for Greek symbols; also passed to \code{\link{latexToken}}
 #' @param label_open,label_close these wrap the entire label; defaults invoke traditional math mode
 #' @param enforce_math whether to enforce math mode for nested expression: \code{\link{latexToken}}
+#' @param script_size three character values, one of which will be appended to token_open for unnested, nested, and multiply-nested contexts
 #' @param ... passed to \code{unrecognized}; see \code{\link{latexToken}}
 #' @examples
 #' library(magrittr)
@@ -52,18 +115,21 @@ as_latex <- function(x, ...)UseMethod('as_latex')
 #' 'var^eta_j' %>% as_spork %>% as_latex
 #' '& % $ # \\_ { } ~ \\^ \\' %>% as_spork %>% as_latex
 #' 'one joule (Omega) ~ 1 kg*m^2./s^2' %>% as_spork %>% as_latex
+#' 'one joule (`Omega`) ~ 1 kg*m^2./s^2' %>% as_spork %>% as_latex
+#' 'one joule (\\`Omega\\`) ~ 1 kg*m^2./s^2' %>% as_spork %>% as_latex
 
 as_latex.spar <- function(
   x,
   newline = getOption('latex_newline','\n'),
-  unrecognized = getOption('latex_unrecognized','latexToken'),
-  token_open = getOption('latex_token_open', '\\textrm{'),
+  unrecognized = getOption('latex_unrecognized',spork::latexToken),
+  token_open = getOption('latex_token_open', '\\textrm{'), ### was \\textrm
   token_close = getOption('latex_token_close','}'),
   math_open = getOption('latex_math_open', '\\mathrm{'),
   math_close = getOption('latex_math_close', '}'),
-  label_open = getOption('latex_label_open', '$'),
-  label_close = getOption('latex_label_close', '$'),
+  label_open = getOption('latex_label_open', '\\('),
+  label_close = getOption('latex_label_close', '\\)'),
   enforce_math = getOption('latex_enforce_math',TRUE),
+  script_size = getOption('latex_script_size', c('','\\scriptsize ','\\tiny ')),
   ...
 ){
   # the latex of a spork is the sequential
@@ -76,16 +142,34 @@ as_latex.spar <- function(
   # which escapes metacharacters and
   # names of Greek letters, but renders other
   # tokens literally.
-
+  
+  stopifnot(is.character(script_size), length(script_size) == 3)
   closers <- character(0)
+  
+  # appends appropriate script size to token_open
+  tokenOpen <- function(token_open, closers, script_size){
+    stopifnot(is.character(token_open), length(token_open) == 1)
+    stopifnot(is.character(closers))
+    stopifnot(is.character(script_size), length(script_size) == 3)
+    level <- length(closers) + 1
+    if(level > 3) level <- 3
+    style <- script_size[[level]]
+    x <- paste0(token_open, style)
+    return(x)
+  }
+  
   active <- FALSE
   if(length(x)==0)return(structure(x, class = union('latex', class(x))))
   if(identical(x, ''))return(structure(x, class = union('latex', class(x))))
   base <- ''
+  greek <- as.character(greek())
+  ungreek <- paste0('`', greek, '`')
+  greek <- paste0('\\b', greek, '\\b') # only at boundaries
   explicit <- c(
-    '[\\][n]', '\\s+',
+    '[\\][n]','\\s+','#+',
     '[*]','[.]','[_]','\\^',
-    '[\\][*]','[\\][.]','[\\][_]','[\\]\\^'
+    '[\\][*]','[\\][.]','[\\][_]','[\\]\\^',
+    greek, ungreek, '[\\][`]'
   )
   for(token in x){
     m <- sapply(explicit, function(pattern)position(token, pattern))
@@ -95,7 +179,7 @@ as_latex.spar <- function(
       token <- fun(
         token,
         unrecognized = unrecognized,
-        token_open = token_open,
+        token_open = tokenOpen(token_open, closers, script_size),
         token_close = token_close,
         math_open = math_open,
         math_close = math_close,
@@ -125,7 +209,22 @@ as_latex.spar <- function(
           base <- paste0(base, newline)
       }
       if(p == '\\s+'){
-        token <- paste0(token_open,token,token_close)
+        token <- paste0(tokenOpen(token_open, closers, script_size),token,token_close)
+        if(active){
+          base <- paste0(base, ' ', token)
+        }else{
+          if(grepl('[]}]$',base)){ # not empty nest
+            base <- paste0(base, ' ', token)
+            active <- TRUE
+          }else{ # empty nest or start of line
+            base <- paste0(base, token)
+            active <- TRUE
+          }
+        }
+      }
+      if(p == '#+'){
+        token <- gsub('#','\\\\#',token)
+        token <- paste0(tokenOpen(token_open, closers, script_size),token,token_close)
         if(active){
           base <- paste0(base, ' ', token)
         }else{
@@ -139,7 +238,7 @@ as_latex.spar <- function(
         }
       }
       if(p == '[\\][*]'){
-        token <- paste0(token_open, '*', token_close)
+        token <- paste0(tokenOpen(token_open, closers, script_size), '*', token_close)
         if(active){
           base <- paste0(base, ' ', token)
         }else{
@@ -148,7 +247,7 @@ as_latex.spar <- function(
         }
       }
       if(p == '[\\][.]'){
-        token <- paste0(token_open, '.', token_close)
+        token <- paste0(tokenOpen(token_open, closers, script_size), '.', token_close)
         if(active){
           base <- paste0(base, ' ', token)
         }else{
@@ -157,7 +256,7 @@ as_latex.spar <- function(
         }
       }
       if(p == '[\\][_]'){
-        token <- paste0(token_open, '\\_', token_close)
+        token <- paste0(tokenOpen(token_open, closers, script_size), '\\_', token_close)
         if(active){
           base <- paste0(base, ' ', token)
         }else{
@@ -166,7 +265,7 @@ as_latex.spar <- function(
         }
       }
       if(p == '[\\]\\^'){
-        token <- paste0(token_open,"{\\textasciicircum}",token_close)
+        token <- paste0(tokenOpen(token_open, closers, script_size),"{\\textasciicircum}",token_close)
         if(active){
           base <- paste0(base, ' ', token)
         }else{
@@ -204,31 +303,88 @@ as_latex.spar <- function(
       if(p == '[_]'){
         closers <- c('}', closers)
         if(active){
-          base <- paste0(base,"_{")
+          base <- paste0(base,"{}_{")
           active <- FALSE
         }else{
           if(!grepl('[]}]$', base)){
             # must have something to subscript
-            base <- paste0(base, "~_{")
+            # https://en.wikipedia.org/wiki/Whitespace_character
+            base <- paste0(base, "{}_{")  # 0.2.6 formerly ~_{ then \\,_{
           }else{
-            base <- paste0(base, "~_{")
+            base <- paste0(base, "{}_{")  # 0.2.6 formerly ~_{ then \\,_{
           }
         }
       }
       if(p == '\\^'){
         closers <- c('}', closers)
         if(active){
-          base <- paste0(base, "^{")
+          base <- paste0(base, "{}^{")  # 0.3.0 was "^{"
           active <- FALSE
         }else{
           if(!grepl('[]}]$', base)){
             # must have something to superscript
-            base <- paste0(base, "~^{")
+            base <- paste0(base, "{}^{") # 0.2.6 formerly ~^{ then \\,{
           }else{
-            base <- paste0(base, "~^{")
+            base <- paste0(base, "{}^{") # 0.2.6 formerly ~^{ then \\,{
           }
         }
       }
+      if(p %in% greek){
+        # as of 0.2.6:
+        # if we got here, the search term p
+        # is literally one of spork greek vocabulary words.
+        # These were formerly handled by latexToken,
+        # but are now unreachable there since they are 
+        # 'recognized' during parsing.
+        # Calling as_latex() gives us the tex code
+        # for the character of interest.
+        # We reproduce here the openers and closers
+        # (borrowed from latexToken)
+        # that are needed to place these in math mode.
+        # i.e.: \\alpha -> $\\mathrm{\\alpha}}$
+        
+        mathopen <- math_open
+        mathclose <- math_close
+        if(enforce_math){
+          mathopen <- paste0(label_open,mathopen)
+          mathclose <- paste0(mathclose, label_close)
+        }
+
+        token <- as_latex(as_greek(token))
+        
+        token <- paste0(
+          tokenOpen(token_open, closers, script_size), 
+            mathopen,
+              token, 
+            mathclose,
+          token_close
+        )
+        if(active){
+          base <- paste0(base, ' ', token)
+        }else{
+          base <- paste0(base, token)
+          active <- TRUE
+        }
+      }
+      if(p %in% ungreek){
+        token <- gsub('`','',token)
+        token <- paste0(tokenOpen(token_open, closers, script_size), token, token_close)
+        if(active){
+          base <- paste0(base, token)
+        }else{
+          base <- paste0(base, token)
+          active <- TRUE
+        }
+      }
+      if(p == '[\\][`]'){
+        token <- paste0(tokenOpen(token_open, closers, script_size), '`', token_close)
+        if(active){
+          base <- paste0(base, ' ', token)
+        }else{
+          base <- paste0(base, ' ', token)
+          active <- TRUE
+        }
+      }
     }
   }
   # use of %.% can leave a dangling operator.
@@ -277,57 +433,63 @@ as_latex.spar <- function(
 latexToken <- function(
   x,
   #unrecognized = latexToken,
-  token_open = getOption('latex_token_open', '\\textrm{'),
+  token_open = getOption('latex_token_open', '\\textrm{'), ### was \\textrm
   token_close = getOption('latex_token_close','}'),
   math_open = getOption('latex_math_open', '\\mathrm{'),
   math_close = getOption('latex_math_close', '}'),
-  label_open = getOption('latex_label_open', '$'),
-  label_close = getOption('latex_label_close', '$'),
+  label_open = getOption('latex_label_open', '\\('),
+  label_close = getOption('latex_label_close', '\\)'),
   enforce_math = getOption('latex_enforce_math',TRUE),
   ...
 ){
-  special <- c(  '&',  '%',  '$',  '#',  '_',  '{',  '}','~',                '^',               '\\'             ) # special in latex
-  replace <- c('\\&','\\%','\\$','\\#','\\_','\\{','\\}','${\\sim}$','{\\textasciicircum}','{\\textbackslash}')      # use in latex
-  greek <- c(
-    'alpha','beta','gamma','delta','epsilon','zeta',
-    'eta','theta','iota','kappa','lambda','mu',
-    'nu','xi','omicron','pi','rho','sigma','tau',
-    'upsilon','phi','chi','psi','omega'
-  )
-  # https://www.overleaf.com/learn/latex/List_of_Greek_letters_and_math_symbols
-  names(greek) <- c(
-    '\\alpha','\\beta','\\gamma','\\delta','\\epsilon','\\zeta',
-    '\\eta','\\theta','\\iota','\\kappa','\\lambda','\\mu',
-    '\\nu','\\xi',
-    'o',
-    '\\pi','\\rho','\\sigma','\\tau',
-    '\\upsilon','\\phi','\\chi','\\psi','\\omega'
-  )
-  Greek <- c(
-    'Alpha','Beta','Gamma','Delta','Epsilon','Zeta',
-    'Eta','Theta','Iota','Kappa','Lambda','Mu',
-    'Nu','Xi','Omicron','Pi','Rho','Sigma','Tau',
-    'Upsilon','Phi','Chi','Psi','Omega'
-  )
-  names(Greek) <- c(
-    'A','B','\\Gamma','\\Delta','E','Z',
-    'H','\\Theta','I','K','\\Lambda','M',
-    'N','\\Xi','O','\\Pi','P','\\Sigma','T',
-    '\\Upsilon','\\Phi','X','\\Psi','\\Omega'
-  )
-  extra <- c( # these are things you can say in plotmath
-    'Upsilon1','varepsilon','omega1',
-    'theta1', 'phi1', 'sigma1',
-    'vartheta','varphi','varsigma',
-    'stigma', 'varrho','varpi'
-  )
-
-  names(extra) <- c( # these are things you can say in tex
-    '\\Upsilon','\\varepsilon','\\varpi',
-    '\\vartheta','\\varphi','\\varsigma',
-    '\\vartheta','\\varphi','\\varsigma',
-    '\\varsigma', '\\varrho','\\varpi'
-  )
+  special <- c(  '&',  '%',  '$',  '#',  '_',  '{',  '}','~', '^','\\') # special in latex
+  replace <- c('\\&','\\%','\\$','\\#','\\_','\\{','\\}','\\({\\sim}\\)','{\\textasciicircum}','{\\textbackslash}')      # use in latex
+  # greek <- c(
+  #   'alpha','beta','gamma','delta',
+  #   'epsilon','zeta','eta','theta',
+  #   'iota','kappa','lambda','mu',
+  #   'nu','xi','omicron','pi',
+  #   'rho','sigma','tau','upsilon',
+  #   'phi','chi','psi','omega'
+  # )
+  # # https://www.overleaf.com/learn/latex/List_of_Greek_letters_and_math_symbols
+  # names(greek) <- c(
+  #   '\\alpha','\\beta','\\gamma','\\delta','\\epsilon','\\zeta',
+  #   '\\eta','\\theta','\\iota','\\kappa','\\lambda','\\mu',
+  #   '\\nu','\\xi',
+  #   'o',
+  #   '\\pi','\\rho','\\sigma','\\tau',
+  #   '\\upsilon','\\phi','\\chi','\\psi','\\omega'
+  # )
+  # Greek <- c(
+  #   'Alpha','Beta','Gamma','Delta',
+  #   'Epsilon','Zeta','Eta','Theta',
+  #   'Iota','Kappa','Lambda','Mu',
+  #   'Nu','Xi','Omicron','Pi',
+  #   'Rho','Sigma','Tau','Upsilon',
+  #   'Phi','Chi','Psi','Omega'
+  # )
+  # names(Greek) <- c(
+  #   'A','B','\\Gamma','\\Delta',
+  #   'E','Z','H','\\Theta',
+  #   'I','K','\\Lambda','M',
+  #   'N','\\Xi','O','\\Pi',
+  #   'P','\\Sigma','T','\\Upsilon',
+  #   '\\Phi','X','\\Psi','\\Omega'
+  # )
+  # extra <- c( # these are things you can say in plotmath
+  #   'Upsilon1','varepsilon','omega1',
+  #   'theta1', 'phi1', 'sigma1',
+  #   'vartheta','varphi','varsigma',
+  #   'stigma', 'varrho','varpi'
+  # )
+  # 
+  # names(extra) <- c( # these are things you can say in tex
+  #   '\\Upsilon','\\varepsilon','\\varpi',
+  #   '\\vartheta','\\varphi','\\varsigma',
+  #   '\\vartheta','\\varphi','\\varsigma',
+  #   '\\varsigma', '\\varrho','\\varpi'
+  # )
 
   # escape <- function(x,pattern,replace)sub(
   #   fixed = TRUE,
@@ -363,7 +525,9 @@ latexToken <- function(
   x <- output
 
   ### greek
-  nms <- c(greek, Greek, extra)
+  # nms <- c(greek, Greek, extra)
+  nms <- greek()
+  names(nms) <- as_latex(greek())
   input <- x
   output <- ''
   while(nchar(input)){
@@ -442,25 +606,6 @@ as_latex.spork <- function(x, ...){
   y
 }
 
-#' Convert Default to Latex
-#'
-#' Converts anything to latex by adding class 'latex'.
-#'
-#' @export
-#' @param x object
-#' @param ... ignored
-#' @return latex
-#' @family latex
-#' @family spork
-#' @family interface
-#' @examples
-#' as_latex('text')
-
-as_latex.default <- function(x, ...){
-  class(x) <- union('latex', class(x))
-  x
-}
-
 #' Subset Latex
 #'
 #' Subsets latex, retaining class.
